/*
 * QuantumLeapUsingQuickSearchShiftAndBorderArray.h
 *
 *  Created on: 27. 1. 2020
 *      Author: Jan Travnicek
 */

#ifndef _QUICK_SEARCH_USING_QUICK_SEARCH_SHIFT_AND_BORDER_ARRAY_H_
#define _QUICK_SEARCH_USING_QUICK_SEARCH_SHIFT_AND_BORDER_ARRAY_H_

#include <alib/set>
#include <alib/map>

#include <tree/ranked/PrefixRankedTree.h>
#include <tree/ranked/PrefixRankedPattern.h>

#include <tree/properties/ReversedBadCharacterShiftTable.h>
#include <tree/properties/BorderArrayNaive.h>
#include <tree/properties/SubtreeJumpTable.h>
#include <tree/exact/ForwardOccurrenceTest.h>

#include <ext/algorithm>

namespace arbology {

namespace exact {

/**
 * Implementation of DeadZone matching using bcs as shifting method to both directions
 */
class QuantumLeapUsingQuickSearchShiftAndBorderArray {
public:
	/**
	 * Search for pattern in a linearised tree.
	 * @return set set of occurences
	 */
	template < class SymbolType >
	static ext::set < unsigned > match ( const tree::PrefixRankedTree < SymbolType > & subject, const tree::PrefixRankedPattern < SymbolType > & pattern, size_t z );
};

template < class SymbolType >
ext::set < unsigned > QuantumLeapUsingQuickSearchShiftAndBorderArray::match ( const tree::PrefixRankedTree < SymbolType > & subject, const tree::PrefixRankedPattern < SymbolType > & pattern, size_t z ) {
	ext::set < unsigned > occ;
	ext::vector < int > subjectSubtreeJumpTable = tree::properties::SubtreeJumpTable::compute ( subject );
	ext::vector < size_t > construct = tree::properties::BorderArray::construct ( pattern );
	ext::map < common::ranked_symbol < SymbolType >, size_t > bqss = tree::properties::ReversedQuickSearchShiftTable::bcs ( pattern ); // NOTE: the subjects alphabet must be a subset or equal to the pattern
	for ( const common::ranked_symbol < SymbolType > & symbol : pattern.getAlphabet ( ) ) {
		bqss [ symbol ] = z - bqss [ symbol ];
	}

	size_t Spos = pattern.getContent ( ).size ( );
	for ( size_t i = 0; i < pattern.getContent ( ).size ( ); ++ i ) {
		if ( pattern.getContent ( ) [ i ] == pattern.getSubtreeWildcard ( ) ) {
			Spos = i;
			break;
		}
	}

	size_t haystack_offset = 0;
	size_t j = 0;

	while ( haystack_offset + pattern.getContent ( ).size ( ) <= subject.getContent ( ).size ( ) ) {
		j = tree::exact::ForwardOccurrenceTest::occurrence ( subject, subjectSubtreeJumpTable, pattern, haystack_offset, j );

		 // match was found
		if ( j >= pattern.getContent ( ).size ( ) ) occ.insert ( haystack_offset );

		if ( haystack_offset + pattern.getContent().size() == subject.getContent().size() ) { // this is needed only because there is no terminating character in the subject ...
			break; // Here we don't do any more shifts if the pattern is already aligned at the utter end of the text
		}

		size_t shf;
		if ( j == 0 ) {
			shf = 1;
		} else {
			size_t shift = j - construct [ j ];
			 // shift heuristics
			shf = shift;
			j = ext::min ( Spos, j ) - shift;
		}

		size_t shb = z;
		if ( haystack_offset + z - 1 < subject.getContent ( ).size ( ) ) // this condition is needed because at worst MAX ( z - m, 0 ) additional characters are needed in the subject after its end
			shb = bqss [ subject.getContent ( ) [ haystack_offset + z - 1 ] ];
		if ( shf > shb ) {
			haystack_offset += z;
			j = 0;
		} else {
			haystack_offset += shf;
		}
	}

	return occ;
}

} /* namespace exact */

} /* namespace arbology */

#endif /* _QUICK_SEARCH_USING_QUICK_SEARCH_SHIFT_AND_BORDER_ARRAY_H_ */
