/*
 * ReversedQuickSearchShiftTable.h
 *
 *  Created on: 24. 1. 2020
 *      Author: Jan Travnicek
 */

#ifndef _STRINGOLOGY_REVERSED_QUICK_SEARCH_SHIFT_TABLE_H_
#define _STRINGOLOGY_REVERSED_QUICK_SEARCH_SHIFT_TABLE_H_

#include <alib/set>
#include <alib/map>

#include <string/LinearString.h>

namespace string {

namespace properties {

/**
 * Computation of BCS table for BMH from MI(E+\eps)-EVY course 2014
 * To get rid of zeros in BCS table we ignore last haystack character
 */
class ReversedQuickSearchShiftTable {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	template < class SymbolType >
	static ext::map < SymbolType, size_t > rqss ( const string::LinearString < SymbolType > & pattern );

};

template < class SymbolType >
ext::map < SymbolType, size_t > ReversedQuickSearchShiftTable::rqss ( const string::LinearString < SymbolType > & pattern ) {
	const ext::set < SymbolType > & alphabet = pattern.getAlphabet ( );
	ext::map < SymbolType, size_t > bcs;

	 /* Initialization of BCS to the length of the needle. */
	for ( const auto & symbol : alphabet )
		bcs.insert ( std::make_pair ( symbol, pattern.getContent ( ).size ( ) + 1 ) );

	 /* Filling out BCS, ignoring first character. */
	for ( ssize_t i = pattern.getContent ( ).size ( ) - 1; i >= 0; i-- )
		bcs[pattern.getContent ( ).at ( i )] = i + 1;

	return bcs;
}

} /* namespace properties */

} /* namespace string */

#endif /* _STRINGOLOGY_REVERSED_QUICK_SEARCH_SHIFT_TABLE_H_ */
