/*
 * QuantumLeapUsingQuickSearchShift.h
 *
 *  Created on: 24. 1. 2020
 *      Author: Jan Travnicek
 */

#ifndef _QUICK_SEARCH_USING_QUICK_SEARCH_SHIFT_H_
#define _QUICK_SEARCH_USING_QUICK_SEARCH_SHIFT_H_

#include <alib/set>
#include <alib/map>

#include <string/LinearString.h>

#include <string/properties/QuickSearchShiftTable.h>
#include <string/properties/ReversedQuickSearchShiftTable.h>

namespace stringology {

namespace exact {

/**
 * Implementation of DeadZone matching using bcs as shifting method to both directions
 */
class QuantumLeapUsingQuickSearchShift {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	template < class SymbolType >
	static ext::set < unsigned > match ( const string::LinearString < SymbolType > & string, const string::LinearString < SymbolType > & pattern, size_t z );
};

template < class SymbolType >
ext::set < unsigned > QuantumLeapUsingQuickSearchShift::match ( const string::LinearString < SymbolType > & string, const string::LinearString < SymbolType > & pattern, size_t z ) {
	ext::set < unsigned > occ;
	ext::map < SymbolType, size_t > fqss = string::properties::QuickSearchShiftTable::qss ( pattern ); // NOTE: the subjects alphabet must be a subset or equal to the pattern
	ext::map < SymbolType, size_t > bqss = string::properties::ReversedQuickSearchShiftTable::rqss ( pattern ); // NOTE: the subjects alphabet must be a subset or equal to the pattern
	for ( const SymbolType & symbol : pattern.getAlphabet ( ) ) {
		bqss [ symbol ] = z - bqss [ symbol ];
	}

	size_t haystack_offset = 0;

	while ( haystack_offset + pattern.getContent ( ).size ( ) <= string.getContent ( ).size ( ) ) {
		size_t i = 0;
		while ( i < pattern.getContent ( ).size ( ) && string.getContent ( ) [ haystack_offset + i ] == pattern.getContent ( ) [ i ] )
			i ++;

		 // Yay, there is match!!!
		if ( i == pattern.getContent ( ).size ( ) ) occ.insert ( haystack_offset );

		if ( haystack_offset + pattern.getContent().size() == string.getContent().size() ) { // this is needed only because there is no terminating character in the string ...
			break; // Here we don't do any more shifts if the pattern is already aligned at the utter end of the text
		}

		size_t shf = fqss [ string.getContent ( ) [ haystack_offset + pattern.getContent ( ).size ( ) ] ];

		size_t shb = z;
		if ( haystack_offset + z - 1 < string.getContent ( ).size ( ) ) // this condition is needed because at worst MAX ( z - m, 0 ) additional characters are needed in the subject after its end
			shb = bqss [ string.getContent ( ) [ haystack_offset + z - 1 ] ];
		haystack_offset += shf > shb ? z : shf;
	}

	return occ;
}

} /* namespace exact */

} /* namespace stringology */

#endif /* _QUICK_SEARCH_USING_QUICK_SEARCH_SHIFT_H_ */
