/*
 * ForwardOccurrenceTest.h
 *
 *  Created on: 31. 3. 2018
 *	  Author: Jan Travnicek
 */

#ifndef FORWARD_OCCURRENCE_TEST_H_
#define FORWARD_OCCURRENCE_TEST_H_

#include <climits>

#include <tree/ranked/PrefixRankedTree.h>
#include <tree/ranked/PrefixRankedPattern.h>
#include <tree/ranked/PrefixRankedNonlinearPattern.h>
#include <tree/ranked/PrefixRankedBarTree.h>
#include <tree/ranked/PrefixRankedBarPattern.h>
#include <tree/ranked/PrefixRankedBarNonlinearPattern.h>

namespace tree {

namespace exact {

class ForwardOccurrenceTest {
public:
	template < class SymbolType >
	static size_t occurrence ( const PrefixRankedBarTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const PrefixRankedBarTree < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition = 0);
	template < class SymbolType >
	static size_t occurrence ( const PrefixRankedBarTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const PrefixRankedBarPattern < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition = 0 );
	template < class SymbolType >
	static size_t occurrence ( const PrefixRankedBarTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const tree::PrefixRankedBarTree < unsigned > & repeats, const PrefixRankedBarNonlinearPattern < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition = 0);

	template < class SymbolType >
	static size_t occurrence ( const PrefixRankedTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const PrefixRankedTree < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition = 0);
	template < class SymbolType >
	static size_t occurrence ( const PrefixRankedTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const PrefixRankedPattern < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition =0 );
	template < class SymbolType >
	static size_t occurrence ( const PrefixRankedTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const tree::PrefixRankedTree < unsigned > & repeats, const PrefixRankedNonlinearPattern < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition = 0 );
};

template < class SymbolType >
size_t ForwardOccurrenceTest::occurrence ( const PrefixRankedBarTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const PrefixRankedBarTree < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition ) {
	return occurrence ( subject, subjectSubtreeJumpTable, tree::PrefixRankedBarPattern < SymbolType > ( pattern ), subjectPosition, patternPosition );
}

template < class SymbolType >
size_t ForwardOccurrenceTest::occurrence ( const PrefixRankedBarTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const PrefixRankedBarPattern < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition ) {
	while ( ( patternPosition < pattern.getContent ( ).size ( ) ) && ( subjectPosition < subject.getContent ( ).size ( ) ) ) {
		if ( subject.getContent ( )[subjectPosition] == pattern.getContent ( )[patternPosition] ) {
			 // match of symbol
			subjectPosition = subjectPosition + 1;
			patternPosition = patternPosition + 1;
		} else if ( ( pattern.getContent ( )[patternPosition] == pattern.getSubtreeWildcard ( ) ) && ( ! subject.getBars ( ).count ( subject.getContent ( )[subjectPosition] ) ) ) { //the second part of the condition is needed to handle S |S
			 // match of variable with subtree
			subjectPosition = subjectSubtreeJumpTable[subjectPosition];
			patternPosition = patternPosition + 2;
		} else {
			break;
		}
	}

	return patternPosition;
}

template < class SymbolType >
size_t ForwardOccurrenceTest::occurrence ( const PrefixRankedBarTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const tree::PrefixRankedBarTree < unsigned > & repeats, const PrefixRankedBarNonlinearPattern < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition ) {

	 // to represent state of variable to subtree repeat
	ext::map < common::ranked_symbol < SymbolType >, unsigned > variablesSetting;

	while ( ( patternPosition < pattern.getContent ( ).size ( ) ) && ( subjectPosition < subject.getContent ( ).size ( ) ) ) {
		if ( subject.getContent ( )[subjectPosition] == pattern.getContent ( )[patternPosition] ) {
			 // match of symbol
			subjectPosition = subjectPosition + 1;
			patternPosition = patternPosition + 1;
		} else if ( ( pattern.getContent ( )[patternPosition] == pattern.getSubtreeWildcard ( ) || pattern.getNonlinearVariables ( ).count ( pattern.getContent ( )[patternPosition] ) ) && ( ! subject.getBars ( ).count ( subject.getContent ( )[subjectPosition] ) ) ) { //the second part of the condition is needed to handle S |S
			 // check nonlinear variable
			if ( pattern.getNonlinearVariables ( ).count ( pattern.getContent ( )[ patternPosition ] ) ) {
				auto setting = variablesSetting.find ( pattern.getContent ( )[ patternPosition ] );

				if ( setting != variablesSetting.end ( ) && repeats.getContent ( )[ subjectPosition ].getSymbol ( ) != setting->second )
					break;

				variablesSetting.insert ( std::make_pair ( pattern.getContent ( )[ patternPosition ], repeats.getContent( )[ subjectPosition ].getSymbol ( ) ) );
			}

			 // match of variable with subtree
			subjectPosition = subjectSubtreeJumpTable[subjectPosition];
			patternPosition = patternPosition + 2;
		} else {
			break;
		}
	}

	return patternPosition;
}

template < class SymbolType >
size_t ForwardOccurrenceTest::occurrence ( const PrefixRankedTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const PrefixRankedTree < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition ) {
	return occurrence ( subject, subjectSubtreeJumpTable, tree::PrefixRankedPattern < SymbolType > ( pattern ), subjectPosition, patternPosition );
}

template < class SymbolType >
size_t ForwardOccurrenceTest::occurrence ( const PrefixRankedTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const PrefixRankedPattern < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition ) {
	while ( ( patternPosition < pattern.getContent ( ).size ( ) ) && ( subjectPosition < subject.getContent ( ).size ( ) ) ) {
		if ( subject.getContent ( )[subjectPosition] == pattern.getContent ( )[patternPosition] )
			 // match of symbol
			subjectPosition = subjectPosition + 1;
		else if ( pattern.getContent ( )[patternPosition] == pattern.getSubtreeWildcard ( ) )
			 // match of variable with subtree
			subjectPosition = subjectSubtreeJumpTable[subjectPosition];
		else
			break;

		patternPosition = patternPosition + 1;
	}

	return patternPosition;
}

template < class SymbolType >
size_t ForwardOccurrenceTest::occurrence ( const PrefixRankedTree < SymbolType > & subject, const ext::vector < int > & subjectSubtreeJumpTable, const tree::PrefixRankedTree < unsigned > & repeats, const PrefixRankedNonlinearPattern < SymbolType > & pattern, size_t subjectPosition, size_t patternPosition ) {
	 // to represent state of variable to subtree repeat
	ext::map < common::ranked_symbol < SymbolType >, unsigned > variablesSetting;

	while ( ( patternPosition < pattern.getContent ( ).size ( ) ) && ( subjectPosition < subject.getContent ( ).size ( ) ) ) {
		if ( subject.getContent ( )[subjectPosition] == pattern.getContent ( )[patternPosition] )
			 // match of symbol
			subjectPosition = subjectPosition + 1;
		else if ( pattern.getContent ( )[patternPosition] == pattern.getSubtreeWildcard ( ) || pattern.getNonlinearVariables ( ).count ( pattern.getContent ( )[ patternPosition ] ) ) {
			 // check nonlinear variable
			if ( pattern.getNonlinearVariables ( ).count ( pattern.getContent ( )[ patternPosition ] ) ) {
				auto setting = variablesSetting.find ( pattern.getContent ( )[ patternPosition ] );

				if ( setting != variablesSetting.end ( ) && repeats.getContent ( )[ subjectPosition ].getSymbol ( ) != setting->second )
					break;

				variablesSetting.insert ( std::make_pair ( pattern.getContent ( )[ patternPosition ], repeats.getContent( )[ subjectPosition ].getSymbol ( ) ) );
			}

			 // match of variable with subtree
			subjectPosition = subjectSubtreeJumpTable[subjectPosition];
		} else
			break;

		patternPosition = patternPosition + 1;
	}

	return patternPosition;
}

} /* namespace exact */

} /* namespace tree */

#endif /* FORWARD_OCCURRENCE_TEST_H_ */
